from typing import TypedDict
from dahua.rpc import RPC


class LoginParams(TypedDict):
    """Parameters for login."""

    userName: str
    password: str
    clientType: str
    authorityType: str
    passwordType: str


class GlobalRPC(RPC):
    def __init__(self, client: "DahuaRpc") -> None:
        super().__init__(client=client, parent="global")

    def login(self, params: LoginParams, **kwargs) -> dict:
        """Get the software version of the MagicBox."""
        return self._send(
            function="login", endpoint="RPC2_Login", params=params, **kwargs
        )

    def logout(self) -> None:
        self._send(function="logout")



from time import sleep
from dataclasses import dataclass

from dahua.client import DahuaRpc
from dahua.utils.logger import logger


@dataclass
class Upgrade:
    client: "DahuaRpc"

    def upgrade(self, firmware_path: str) -> bool:
        """Upgrade the firmware of the device."""
        logger.info("Starting firmware upgrade...")

        # Upload the firmware
        logger.info("Uploading firmware")
        if not self._upload_firmware(firmware_path):
            logger.error("Failed to upload firmware.")
            return False

        # Check upgrade progress
        if not self._check_progress():
            logger.error("Upgrade failed")
            return False

        logger.info("Firmware upgrade completed successfully.")
        return True

    def _upload_firmware(self, firmware_path: str) -> bool:
        cookies = {"DWebClientSessionID": self.client.session_id}

        with open(firmware_path, "rb") as f:
            files = {"fileupload": f}
            self.client.request_id += 1
            response = self.client.session.post(
                f"{self.client.base_url}/RPC2_Upgrade", files=files, cookies=cookies
            )

            return "sonia upgrade successfully" in response.text

    def _check_progress(self) -> bool:
        state = "Upgrading"

        while state == "Upgrading":
            update = self.client.upgrader.get_state()
            state = update.get("State")

            logger.info(f"Upgrade progress: {update.get('Progress')}%")
            logger.debug(f"Upgrade state: {update}")
            sleep(5)

        logger.debug(update)
        return state != "Invalid"



import hashlib


def generate_password_hash(
    username: str, password: str, realm: str, random_value: str
) -> str:
    """Generate a password hash using MD5 hashing algorithm."""
    pwd_phrase = f"{username}:{realm}:{password}".encode("utf-8")
    pwd_hash = hashlib.md5(pwd_phrase).hexdigest().upper()
    pass_phrase = f"{username}:{random_value}:{pwd_hash}".encode("utf-8")
    return hashlib.md5(pass_phrase).hexdigest().upper()



from dahua.rpc import RPC


class ConsoleRPC(RPC):
    def __init__(self, client: "DahuaRpc") -> None:
        super().__init__(client=client, parent="console")

    def run_cmd(self, command: str) -> dict:
        return self._send(function="runCmd", params={"command": command})



class DahuaRequestError(Exception):
    """Exception raised for request errors."""


class DahuaMethodNotSupported(Exception):
    """Methods that a function parent does not support"""


def code_to_error(code: int):
    """
    ERROR_CODE_NOT_SET = 268959743,
    INTERFACE_NOT_FOUND = 268632064,
    METHOD_NOT_FOUND = 268894210,
    REQUEST_INVALID = 268894209,
    REQUEST_INVALID_PARAM = 268894211,
    SESSION_INVALID = 28763750,

    USER_NOT_VALID = 268632070,
    PASSWORD_NOT_VALID = 268632071,
    IN_BLACK_LIST = 268632073,
    HAS_BEEN_USED = 268632074,
    HAS_BEEN_LOCKED = 268632081,

    BUSY = 268632075,
    """
    # TODO create exceptions for each and map



from typing import Any, TypedDict
from dahua.exceptions import DahuaRequestError, DahuaMethodNotSupported


class ListMethodResponse(TypedDict):
    method: list[str]


class RPC:
    def __init__(self, client: "DahuaRpc", parent: str):
        self.client = client
        self.parent = parent

    def _send(self, function: str, **kwargs) -> dict[str, Any]:
        """Send a request to the camera."""
        return self.client._request(method=f"{self.parent}.{function}", **kwargs)

    # ========================================
    # Common Methods
    # ========================================

    def list_method(self) -> ListMethodResponse:
        """Lists all methods of RPC function"""
        try:
            return self._send(function="listMethod").get("params")
        except DahuaRequestError as e:
            raise DahuaMethodNotSupported from e



from dahua.rpc import RPC
from typing import TypedDict


class GetStateResponse(TypedDict):
    File: str
    Progress: int
    State: str
    NoReboot: list[str] | None


class UpgraderRPC(RPC):
    def __init__(self, client: "DahuaRpc") -> None:
        super().__init__(client=client, parent="upgrader")

    def get_state(self) -> GetStateResponse:
        """Get the state of the Upgrader."""
        return self._send(function="getState").get("params", {})

